import { useCallback, useMemo } from 'react';
import BigNumber from 'bignumber.js';
import { AsyncStatus, useAsyncPoll, useAsyncStatuses, useAsyncValue } from '@onomy/react-utils';
import {
  useDetailedBalance,
  useDetailedBalances,
  useOnomyHub,
  useSupportedAssetList,
  useAsset,
} from '@onomy/react-hub';
import { Blockchain, FungibleAsset, FungibleAssetOnChain } from '@onomy/frontend-relay-types';
import {
  DetailedFungibleAssetBalance,
  DetailedFungibleAssetOnChain,
  OnomyHubConstants,
  OnomyHubUtils,
} from '@onomy/hub';
import { OrderResponse } from '@onomy/telescope/dist/codegen/market/order';
import { Pool } from '@onomy/telescope/dist/codegen/market/pool';
import { Member } from '@onomy/telescope/dist/codegen/market/member';
import { Drop } from '@onomy/telescope/dist/codegen/market/drop';
import { OnexClient } from '@onomy/client';

export function useOnexClient() {
  const { hub } = useOnomyHub();
  const onex = hub?.chains.onex?.client;
  if (!onex) return null;
  return onex;
}

export function useOnexPoolList() {
  const onex = useOnexClient();

  return useAsyncPoll(
    useCallback(async () => onex?.poolAll() ?? [], [onex]),
    [] as Awaited<ReturnType<OnexClient['poolAll']>>
  );
}

export function useOnexBurned(): [DetailedFungibleAssetBalance, AsyncStatus] {
  const onex = useOnexClient();
  const chain = onex?.getBlockchain();
  const [assets, assetsStatus] = useSupportedAssetList();

  const [burnedRaw, burnedStatus] = useAsyncPoll(
    useCallback(async () => onex?.burned() ?? null, [onex]),
    null
  );

  const asset = useMemo(() => {
    if (!burnedRaw || !chain?.name) return OnomyHubConstants.UNKNOWN_BALANCE.asset;
    const burnAsset = assets.find(a => a.id === burnedRaw.denom && a.chain.name === chain.name);
    return burnAsset ?? OnomyHubConstants.UNKNOWN_BALANCE.asset;
  }, [assets, burnedRaw, chain?.name]);

  const [amount, amountStatus] = useDetailedBalance(asset, burnedRaw?.amount ?? '0');
  const status = useAsyncStatuses([assetsStatus, burnedStatus, amountStatus]);
  return [amount, status];
}

export function useOnexBurnings(): [DetailedFungibleAssetBalance[], AsyncStatus] {
  const onex = useOnexClient();
  const chain = onex?.getBlockchain();
  const [assets, assetsStatus] = useSupportedAssetList();

  const [burningsRaw, status] = useAsyncPoll(
    useCallback(async () => onex?.burningsAll() ?? [], [onex]),
    [] as Awaited<ReturnType<OnexClient['burningsAll']>>
  );

  const burnings = useMemo(() => {
    if (!chain?.name) return [];
    return burningsRaw
      .map(raw => {
        const asset = assets.find(a => a.id === raw.denom && a.chain.name === chain.name);
        if (!asset) return null;
        return {
          asset,
          amount: BigInt(raw.amount),
        };
      })
      .filter(x => !!x) as DetailedFungibleAssetBalance[];
  }, [burningsRaw, chain?.name, assets]);

  const [amounts, amountStatus] = useDetailedBalances(burnings);
  const combinedStatus = useAsyncStatuses([assetsStatus, status, amountStatus]);
  return [amounts, combinedStatus];
}

export function useOnexPairList() {
  const [pools, poolStatus] = useOnexPoolList();
  const pairs = useMemo(() => pools.map(pool => pool.pair), [pools]);
  return [pairs, poolStatus] as [string[], AsyncStatus];
}

export function useOnexPool({ baseDenom = 'anom', quoteDenom = 'usdt' }) {
  const onex = useOnexClient();
  const pair = useMemo(() => [baseDenom, quoteDenom].sort().join(','), [baseDenom, quoteDenom]);
  const cb = useCallback(async () => {
    try {
      const res = (await onex?.pool(pair)) ?? null;
      if (!res || res.drops === '0') return null;
      return res;
    } catch (e: any) {
      // TODO: need this error handling for some other queries as well
      if (e?.message?.match(/not found/)) return null;
      throw e;
    }
  }, [pair, onex]);

  return useAsyncPoll(cb, null);
}

export function useOnexPairMembers({ baseDenom = 'anom', quoteDenom = 'usdt' }) {
  const onex = useOnexClient();

  const cb = useCallback(async () => {
    try {
      const [memberBase, memberQuote] = await Promise.all([
        onex?.member({
          denomA: quoteDenom,
          denomB: baseDenom,
        }) ?? Promise.resolve(null),
        onex?.member({
          denomA: baseDenom,
          denomB: quoteDenom,
        }) ?? Promise.resolve(null),
      ]);
      return { memberBase, memberQuote };
    } catch (e: any) {
      if (e?.message?.match(/not found/)) return { memberBase: null, memberQuote: null };
      throw e;
    }
  }, [onex, baseDenom, quoteDenom]);

  return useAsyncPoll(cb, { memberBase: null, memberQuote: null });
}

async function includePoolDetails({
  hub,
  onex,
  baseAsset,
  quoteAsset,
  pool,
  memberBase,
  memberQuote,
}: {
  onex: ReturnType<typeof useOnexClient>;
  hub: ReturnType<typeof useOnomyHub>['hub'];
  pool: Pool;
  memberBase: Member;
  memberQuote: Member;
  baseAsset: DetailedFungibleAssetOnChain;
  quoteAsset: DetailedFungibleAssetOnChain;
}) {
  if (!hub) throw new Error('Hub is null');

  const [memberBaseTotal, memberQuoteTotal, leaders] = await Promise.all([
    hub.balances.includeDetails({
      amount: BigInt(memberBase.balance),
      asset: baseAsset,
    }),
    hub.balances.includeDetails({
      amount: BigInt(memberQuote.balance),
      asset: quoteAsset,
    }),
    Promise.all(
      pool.leaders.map(async leader => {
        const amounts = await (async () => {
          const fallback = {
            denom1: baseAsset.id,
            denom2: quoteAsset.id,
            amount1: '0',
            amount2: '0',
          };
          try {
            // TODO: figure out why API is failing here
            return (
              (await onex?.dropsToCoins(baseAsset.id, quoteAsset.id, leader.drops)) ?? fallback
            );
          } catch (e) {
            // eslint-disable-next-line no-console
            console.warn(
              'Error fetching dropsToCoins',
              { pair: pool.pair, drops: leader.drops },
              e
            );
            return fallback;
          }
        })();
        const leaderBaseAmountFixed = amounts.amount1;
        const leaderQuoteAmountFixed = amounts.amount2;

        const [baseAmount, quoteAmount] = await Promise.all([
          hub.balances.includeDetails({
            amount: BigInt(leaderBaseAmountFixed),
            asset: baseAsset,
          }),
          hub.balances.includeDetails({
            amount: BigInt(leaderQuoteAmountFixed),
            asset: quoteAsset,
          }),
        ]);

        return {
          ...leader,
          baseAmount,
          quoteAmount,
        };
      })
    ),
  ]);

  return {
    ...pool,
    baseAsset,
    quoteAsset,
    fiatTotal: memberBaseTotal.fiatValue + memberQuoteTotal.fiatValue,
    fiatCurrency: memberBaseTotal.fiatCurrency,
    leaders,
    memberBase: {
      ...memberBase,
      total: memberBaseTotal,
    },
    memberQuote: {
      ...memberQuote,
      total: memberQuoteTotal,
    },
  };
}

export type OnexDetailedPool = Awaited<ReturnType<typeof includePoolDetails>>;

export function useOnexPoolDetail({
  baseAsset,
  quoteAsset,
}: {
  baseAsset: DetailedFungibleAssetOnChain;
  quoteAsset: DetailedFungibleAssetOnChain;
}): [OnexDetailedPool | null, AsyncStatus] {
  const baseDenom = baseAsset.id;
  const quoteDenom = quoteAsset.id;
  const { hub } = useOnomyHub();
  const onex = useOnexClient();
  const [pool, poolStatus] = useOnexPool({ baseDenom, quoteDenom });
  const [{ memberBase, memberQuote }, membersStatus] = useOnexPairMembers({
    baseDenom,
    quoteDenom,
  });

  const detailedPoolCb = useCallback(async () => {
    if (!pool || !memberBase || !memberQuote || !hub) return null;
    return includePoolDetails({ hub, onex, baseAsset, quoteAsset, pool, memberBase, memberQuote });
  }, [hub, onex, baseAsset, quoteAsset, pool, memberBase, memberQuote]);

  const [detailedPool, detailedPoolStatus] = useAsyncPoll(detailedPoolCb, null);

  const combinedStatus = useAsyncStatuses([poolStatus, membersStatus, detailedPoolStatus]);
  return [detailedPool, combinedStatus];
}

export function useOnexPoolPrice({ baseDenom = 'anom', quoteDenom = 'usdt' }) {
  const onex = useOnexClient();
  const cb = useCallback(async () => {
    try {
      return (await onex?.getPoolDisplayPrice({ baseDenom, quoteDenom })) ?? '0';
    } catch (e: any) {
      if (e?.message?.match(/not found/)) return '0';
      throw e;
    }
  }, [onex, baseDenom, quoteDenom]);

  return useAsyncPoll(cb, '0');
}

export function useFixedAmountFromDisplay(
  displayValue: string | number,
  asset: DetailedFungibleAssetOnChain
) {
  return useMemo(() => {
    const bn = new BigNumber(displayValue);
    if (bn.isNaN()) return '0';
    return bn.shiftedBy(asset.meta.decimalPlaces).toFixed(0);
  }, [displayValue, asset.meta.decimalPlaces]);
}

export function useOnexDetailedPoolPrice({
  baseAsset,
  quoteAsset,
}: {
  baseAsset: DetailedFungibleAssetOnChain;
  quoteAsset: DetailedFungibleAssetOnChain;
}): [DetailedFungibleAssetBalance, AsyncStatus] {
  const baseDenom = baseAsset.id;
  const quoteDenom = quoteAsset.id;
  const [currentPrice, currentPriceStatus] = useOnexPoolPrice({ baseDenom, quoteDenom });
  const currentPriceFixed = useFixedAmountFromDisplay(currentPrice, quoteAsset);
  const [currentPricePerBase, currentPricePerBaseStatus] = useDetailedBalance(
    quoteAsset,
    currentPriceFixed
  );
  const status = useAsyncStatuses([currentPriceStatus, currentPricePerBaseStatus]);
  return [currentPricePerBase, status];
}

export function useOnexLastExecutionPrice({ baseDenom = 'anom', quoteDenom = 'usdt' }) {
  const onex = useOnexClient();
  const cb = useCallback(async () => {
    try {
      if (!baseDenom || !quoteDenom) return '0';
      return (await onex?.getLastTradeDisplayPrice({ baseDenom, quoteDenom })) ?? '0';
    } catch (e: any) {
      if (e?.message?.match(/not found/)) return '0';
      throw e;
    }
  }, [onex, baseDenom, quoteDenom]);

  return useAsyncPoll(cb, '0');
}

export function useOnexDetailedLastExecutionPrice({
  baseAsset,
  quoteAsset,
}: {
  baseAsset: DetailedFungibleAssetOnChain | null;
  quoteAsset: DetailedFungibleAssetOnChain | null;
}): [DetailedFungibleAssetBalance, AsyncStatus] {
  const baseDenom = baseAsset?.id;
  const quoteDenom = quoteAsset?.id;
  const [currentPrice, currentPriceStatus] = useOnexLastExecutionPrice({ baseDenom, quoteDenom });
  const currentPriceFixed = useFixedAmountFromDisplay(
    currentPrice,
    quoteAsset ?? OnomyHubConstants.NOM_ASSET
  );
  const [currentPricePerBase, currentPricePerBaseStatus] = useDetailedBalance(
    quoteAsset,
    currentPriceFixed
  );
  const status = useAsyncStatuses([currentPriceStatus, currentPricePerBaseStatus]);
  return [currentPricePerBase, status];
}

async function includeOrderDetail({
  hub,
  onex,
  order,
  baseAsset,
  quoteAsset,
}: {
  hub: ReturnType<typeof useOnomyHub>['hub'];
  onex: ReturnType<typeof useOnexClient>;
  order: OrderResponse;
  baseAsset: DetailedFungibleAssetOnChain;
  quoteAsset: DetailedFungibleAssetOnChain;
}) {
  if (!hub) throw new Error('Hub is null');
  const direction = order.denomAsk === baseAsset.id ? 'buy' : 'sell';
  const baseAmount = {
    amount:
      direction === 'sell'
        ? BigInt(order.amount)
        : BigInt(
            new BigNumber(order.rate[0]).times(order.amount).dividedBy(order.rate[1]).toFixed(0)
          ),
    asset: baseAsset,
  };

  const quoteAmountSimple = {
    amount:
      direction === 'buy'
        ? BigInt(order.amount)
        : BigInt(
            new BigNumber(order.rate[0]).times(order.amount).dividedBy(order.rate[1]).toFixed(0)
          ),
    asset: quoteAsset,
  };

  const [price, amount, quoteAmount] = await Promise.all([
    onex?.getDisplayPriceFromRatio({
      baseDenom: baseAsset.id,
      baseAmountFixed: order.rate[direction === 'buy' ? 0 : 1],
      quoteDenom: quoteAsset.id,
      quoteAmountFixed: order.rate[direction === 'buy' ? 1 : 0],
    }) ?? Promise.resolve('0'),
    hub!.balances.includeDetails(baseAmount),
    hub!.balances.includeDetails(quoteAmountSimple),
  ]);

  // TODO: room for optimization here since getDisplayPriceFromRation converts to str and we convert back to BN
  const bnPrice = new BigNumber(price).shiftedBy(quoteAsset.meta.decimalPlaces);
  const pricePer = await hub!.balances.includeDetails({
    amount: BigInt(bnPrice.toFixed(0)),
    asset: quoteAsset,
  });

  return {
    ...order,
    amount,
    quoteAmount,
    price,
    pricePer,
    direction,
    baseAsset,
    quoteAsset,
  };
}

export function useOnexOrderHistory({
  baseAsset,
  quoteAsset,
  length = 10,
}: {
  baseAsset: DetailedFungibleAssetOnChain;
  quoteAsset: DetailedFungibleAssetOnChain;
  length?: number;
}) {
  const { hub } = useOnomyHub();
  const onex = useOnexClient();
  const pair = useMemo(
    () => [baseAsset.id, quoteAsset.id].sort().join(','),
    [baseAsset.id, quoteAsset.id]
  );

  const cb = useCallback(async () => {
    try {
      const orders = hub && onex ? await onex.history({ pair, length: `${length}` }) : [];
      return await Promise.all(
        orders.map(order => includeOrderDetail({ hub, onex, baseAsset, quoteAsset, order }))
      );
    } catch (e: any) {
      if (e?.message?.match(/not found/)) return [];
      throw e;
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [baseAsset.id, quoteAsset.id, length, pair, hub, onex]);

  return useAsyncPoll(cb, [] as Awaited<ReturnType<typeof cb>>);
}

export function useOnexMyActiveOrders({
  baseAsset,
  quoteAsset,
  length = 10,
}: {
  baseAsset: DetailedFungibleAssetOnChain;
  quoteAsset: DetailedFungibleAssetOnChain;
  length?: number;
}) {
  const { hub } = useOnomyHub();
  const onex = useOnexClient();
  const pair = useMemo(
    () => [baseAsset.id, quoteAsset.id].sort().join(','),
    [baseAsset.id, quoteAsset.id]
  );

  const cb = useCallback(async () => {
    const orders = hub && onex ? await onex.ordersByOwner() : [];
    return Promise.all(
      orders
        .filter(order => [order.denomAsk, order.denomBid].sort().join(',') === pair)
        .map(order => includeOrderDetail({ hub, onex, baseAsset, quoteAsset, order }))
    );
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [baseAsset.id, quoteAsset.id, pair, length, hub, onex]);

  return useAsyncPoll(cb, [] as Awaited<ReturnType<typeof cb>>);
}

export function useOnexMarketEstimate({ denomAsk = 'usdt', denomBid = 'anom', amountBid = '0' }) {
  const onex = useOnexClient();

  const cb = useCallback(async () => {
    const bnAmount = new BigNumber(amountBid);
    if (!onex || bnAmount.lte(0) || bnAmount.isNaN() || !bnAmount.isFinite()) return '0';
    const resp = await onex.quote({
      denomAsk,
      denomBid,
      denomAmount: denomBid,
      amount: amountBid,
    });

    return resp.amount;
  }, [denomAsk, denomBid, amountBid, onex]);

  return useAsyncPoll(cb, '0');
}

export function useOnexOrderBook({
  baseAsset,
  quoteAsset,
  direction,
  orderType = 'limit',
}: {
  baseAsset: DetailedFungibleAssetOnChain;
  quoteAsset: DetailedFungibleAssetOnChain;
  direction: 'buy' | 'sell';
  orderType?: 'limit';
}) {
  const { hub } = useOnomyHub();
  const onex = useOnexClient();
  const cb = useCallback(async () => {
    // TODO: use index order book rather than individual orders
    const denomA = direction === 'buy' ? baseAsset.id : quoteAsset.id;
    const denomB = direction === 'sell' ? baseAsset.id : quoteAsset.id;
    const book = hub && onex ? await onex.book({ denomA, denomB, orderType }) : [];
    const res = await Promise.all(
      book.map(order => includeOrderDetail({ hub, onex, baseAsset, quoteAsset, order }))
    );
    return direction === 'buy' ? res : res.reverse();
  }, [onex, baseAsset, quoteAsset, direction, hub, orderType]);

  return useAsyncPoll(cb, [] as Awaited<ReturnType<typeof cb>>);
}

function calculateDrops(pool: OnexDetailedPool, rawDrops: Drop[]) {
  const bnBaseMemberAmount = new BigNumber(pool.memberBase.balance);
  const bnQuoteMemberAmount = new BigNumber(pool.memberQuote.balance);
  const bnPoolDrops = new BigNumber(pool.drops);

  return rawDrops.map(drop => {
    const bnProduct = new BigNumber(drop.product);
    const bnDrops = new BigNumber(drop.drops);
    const bnTotalBase = bnBaseMemberAmount.multipliedBy(bnDrops).dividedBy(bnPoolDrops);
    const bnTotalQuote = bnQuoteMemberAmount.multipliedBy(bnDrops).dividedBy(bnPoolDrops);
    const bnPrincipalBase = bnProduct
      .multipliedBy(bnBaseMemberAmount)
      .dividedBy(bnQuoteMemberAmount)
      .sqrt();
    const bnPrincipalQuote = bnProduct
      .multipliedBy(bnQuoteMemberAmount)
      .dividedBy(bnBaseMemberAmount)
      .sqrt();
    const bnRewardBase = bnTotalBase.minus(bnPrincipalBase);
    const bnRewardQuote = bnTotalQuote.minus(bnPrincipalQuote);

    return {
      ...drop,

      bnTotalBase,
      bnTotalQuote,
      bnPrincipalBase,
      bnPrincipalQuote,
      bnRewardBase,
      bnRewardQuote,
    };
  });
}

export type OnexCalculatedDropListType = ReturnType<typeof calculateDrops>;

async function includeDropsDetails(
  hub: ReturnType<typeof useOnomyHub>['hub'],
  pool: OnexDetailedPool,
  calculatedDrops: OnexCalculatedDropListType
) {
  if (!hub) throw new Error('Hub is null');
  return Promise.all(
    calculatedDrops.map(async drop => {
      const [totalBase, totalQuote, principalBase, principalQuote, rewardBase, rewardQuote] =
        await Promise.all([
          hub.balances.includeDetails({
            amount: BigInt(drop.bnTotalBase.toFixed(0)),
            asset: pool.baseAsset,
          }),
          hub.balances.includeDetails({
            amount: BigInt(drop.bnTotalQuote.toFixed(0)),
            asset: pool.quoteAsset,
          }),
          hub.balances.includeDetails({
            amount: BigInt(drop.bnPrincipalBase.toFixed(0)),
            asset: pool.baseAsset,
          }),
          hub.balances.includeDetails({
            amount: BigInt(drop.bnPrincipalQuote.toFixed(0)),
            asset: pool.quoteAsset,
          }),
          hub.balances.includeDetails({
            amount: BigInt(drop.bnRewardBase.toFixed(0)),
            asset: pool.baseAsset,
          }),
          hub.balances.includeDetails({
            amount: BigInt(drop.bnRewardQuote.toFixed(0)),
            asset: pool.quoteAsset,
          }),
        ]);

      const principalFiat = principalBase.fiatValue + principalQuote.fiatValue;
      const rewardFiat = rewardBase.fiatValue + rewardQuote.fiatValue;
      const totalFiat = principalFiat + rewardFiat;

      return {
        ...drop,
        pool,
        totalBase,
        totalQuote,
        principalBase,
        principalQuote,
        rewardBase,
        rewardQuote,
        principalFiat,
        rewardFiat,
        totalFiat,
      };
    })
  );
}

export type OnexDetailedDropListType = Awaited<ReturnType<typeof includeDropsDetails>>;

export function useOnexMyDropsForPool(
  pool: OnexDetailedPool | null
): [OnexDetailedDropListType, AsyncStatus] {
  const { hub } = useOnomyHub();
  const onex = useOnexClient();

  const rawDropsCb = useCallback(async () => {
    if (!onex || !pool?.pair) return [];
    try {
      return await onex.dropOwnerPair(pool.pair);
    } catch (e: any) {
      if (e?.message?.match(/not found/)) return [];
      throw e;
    }
  }, [onex, pool?.pair]);

  const [rawDrops, rawDropsStatus] = useAsyncPoll(
    rawDropsCb,
    [] as Awaited<ReturnType<typeof rawDropsCb>>
  );

  const calculatedDrops = useMemo(() => {
    if (!pool?.pair) return [];
    return calculateDrops(pool, rawDrops);
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [rawDrops, pool?.pair, pool?.drops, pool?.memberBase.balance, pool?.memberQuote.balance]);

  const detailedDropsCb = useCallback(async () => {
    if (!hub || !pool) return [];
    return includeDropsDetails(hub, pool, calculatedDrops);
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [hub, calculatedDrops]);

  const [detailedDrops, detailedDropsStatus] = useAsyncPoll(
    detailedDropsCb,
    [] as Awaited<ReturnType<typeof detailedDropsCb>>
  );

  const status = useAsyncStatuses([rawDropsStatus, detailedDropsStatus]);
  return [detailedDrops, status];
}

export function useOnexMyDrops() {
  const { hub } = useOnomyHub();
  const onex = useOnexClient();

  const dropPairsCb = useCallback(async () => {
    try {
      return (await onex?.dropPairs()) ?? [];
    } catch (e: any) {
      if (e?.message?.match(/not found/)) return [];
      throw e;
    }
  }, [onex]);

  const [allAssets, assetsStatus] = useSupportedAssetList();

  const [rawPairs, rawPairsStatus] = useAsyncPoll(
    dropPairsCb,
    [] as Awaited<ReturnType<typeof dropPairsCb>>
  );

  const allDropsCb = useCallback(async () => {
    if (!onex) return [];
    const onexChain = onex.getBlockchain();
    const onexAssets = allAssets.filter(asset => asset.chain.name === onexChain.name);
    if (!hub || onexAssets.length <= 0) return [];
    return Promise.all(
      rawPairs.map(async pair => {
        const [baseDenom, quoteDenom] = pair.split(','); // TODO: better determine base vs quote

        const baseAsset = onexAssets.find(asset => asset.id === baseDenom);
        const quoteAsset = onexAssets.find(asset => asset.id === quoteDenom);

        if (!baseAsset) throw new Error(`baseAsset not found for ${baseDenom}`);
        if (!quoteAsset) throw new Error(`quoteAsset not found for ${quoteDenom}`);

        const [rawDrops, pool, memberBase, memberQuote] = await Promise.all([
          onex.dropOwnerPair(pair),
          onex.pool(pair),
          onex.member({ denomA: quoteDenom, denomB: baseDenom }),
          onex.member({ denomA: baseDenom, denomB: quoteDenom }),
        ]);
        const detailedPool = await includePoolDetails({
          hub,
          onex,
          baseAsset,
          quoteAsset,
          pool,
          memberBase,
          memberQuote,
        });
        const calculatedDrops = await calculateDrops(detailedPool, rawDrops);
        const drops = await includeDropsDetails(hub, detailedPool, calculatedDrops);

        const basePrincipalAmounts = drops.map(drop => drop.principalBase);
        const quotePrincipalAmounts = drops.map(drop => drop.principalQuote);
        const baseRewardAmounts = drops.map(drop => drop.rewardBase);
        const quoteRewardAmounts = drops.map(drop => drop.rewardQuote);

        const [basePrincipalRaw] = OnomyHubUtils.combineBalances(basePrincipalAmounts);
        const [quotePrincipalRaw] = OnomyHubUtils.combineBalances(quotePrincipalAmounts);
        const [baseRewardRaw] = OnomyHubUtils.combineBalances(baseRewardAmounts);
        const [quoteRewardRaw] = OnomyHubUtils.combineBalances(quoteRewardAmounts);
        const [baseTotalRaw] = OnomyHubUtils.combineBalances([basePrincipalRaw, baseRewardRaw]);
        const [quoteTotalRaw] = OnomyHubUtils.combineBalances([quotePrincipalRaw, quoteRewardRaw]);

        const [basePrincipal, quotePrincipal, baseReward, quoteReward, baseTotal, quoteTotal] =
          await Promise.all([
            hub.balances.includeDetails(basePrincipalRaw),
            hub.balances.includeDetails(quotePrincipalRaw),
            hub.balances.includeDetails(baseRewardRaw),
            hub.balances.includeDetails(quoteRewardRaw),
            hub.balances.includeDetails(baseTotalRaw),
            hub.balances.includeDetails(quoteTotalRaw),
          ]);

        const fiatPrincipal = basePrincipal.fiatValue + quotePrincipal.fiatValue;
        const fiatReward = baseReward.fiatValue + quoteReward.fiatValue;
        const fiatTotal = fiatPrincipal + fiatReward;

        return {
          pool: detailedPool,
          drops,
          basePrincipal,
          quotePrincipal,
          baseReward,
          quoteReward,

          baseTotal,
          quoteTotal,

          fiatPrincipal,
          fiatReward,
          fiatTotal,
        };
      })
    );
  }, [hub, onex, rawPairs, allAssets]);

  return useAsyncPoll(allDropsCb, [] as Awaited<ReturnType<typeof allDropsCb>>);
}
