import React, { createContext, useContext, useCallback, useEffect, useState } from 'react';
import { useOnomyHub } from '@onomy/react-hub';

import { PlaceholderErrorDisplay } from '../../components/PlaceholderErrorDisplay';
import { PlaceholderLoadingIndicator } from '../../components/PlaceholderLoadingIndicator';

function useMnemonicWalletState() {
  const { wallet } = useOnomyHub();
  const [generatedMnemonic, setGeneratedMnemonic] = useState('');

  const connectWallet = useCallback(async () => {
    return Promise.all([
      wallet.onomy.connect(wallet.onomy.wallets[0]),
      wallet.onex.connect(wallet.onex.wallets[0]),
    ]);
  }, [wallet]);

  const saveMnemonic = useCallback(
    async (mnemonic: string) => {
      await wallet.doSave(mnemonic.toLowerCase());
      await connectWallet();
      setGeneratedMnemonic('');
    },
    [wallet, connectWallet]
  );

  const getMnemonic = useCallback(
    async (password?: string) => wallet.getMnemonic(password),
    [wallet]
  );

  const generateMnemonic = useCallback(async () => {
    if (!wallet.isMissing && !generatedMnemonic) return;
    const created = await wallet.doCreate();
    setGeneratedMnemonic(created);
  }, [wallet, generatedMnemonic]);

  const eraseMnemonic = useCallback(async () => {
    await wallet.doErase();
  }, [wallet]);

  useEffect(() => {
    (async () => {
      try {
        if (wallet.isLoaded && !wallet.isMissing) await connectWallet();
      } catch (e) {
        // eslint-disable-next-line no-console
        console.error('wallet load error', e);
      }
    })();
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [wallet.isLoaded]);

  return {
    error: wallet.error,

    isWalletLoaded: !!(wallet.isLoaded || generatedMnemonic),
    isMnemonicSaved: !wallet.isMissing && !generatedMnemonic,

    generatedMnemonic,

    generateMnemonic,
    getMnemonic,
    checkMnemonic: wallet.checkMnemonic,
    saveMnemonic,
    eraseMnemonic,
  };
}

export type MnemonicWalletContextType = ReturnType<typeof useMnemonicWalletState>;

export const MnemonicWalletContext = createContext<MnemonicWalletContextType>(null as any);

export function useMnemonicWallet() {
  const state = useContext(MnemonicWalletContext);
  if (!state) throw new Error('Missing MnemonicWalletContext');
  return state;
}

function defaultRenderLoading() {
  return <PlaceholderLoadingIndicator text="Loading vote form" />;
}

function defaultRenderError(error: any) {
  return <PlaceholderErrorDisplay error={error} />;
}

export function MnemonicWalletProvider({
  children,
  renderLoading,
  renderError = defaultRenderError,
}: {
  children: JSX.Element | JSX.Element[];
  renderLoading?: typeof defaultRenderLoading;
  renderError?: typeof defaultRenderError;
}) {
  const state = useMnemonicWalletState();

  if (state.error) return renderError(state.error);
  if (!state.isWalletLoaded && renderLoading) return renderLoading();
  return <MnemonicWalletContext.Provider value={state}>{children}</MnemonicWalletContext.Provider>;
}
